#include ".\map_win.h"
#include ".\map.h"

// Tile Draw selected Left Button Tile
int mapLeftTile = -1;
// Tile Draw selected Right Button Tile
int mapRightTile = -1;

// Current Editting/Drawing Layer
int mapLayer = 0;

// Map Scroll Speed
int mapScrollSpeed = 1;

// Tileset used to render the map
CRM32Pro_CTile* mapTileset = NULL;

// The Map.
Map* MyMap = NULL;

// Map filename
string MapPath = "test.map";

// Map Window Status
string mapStatus;

const int getMapScrollSpeed(void)
{
	return mapScrollSpeed;
}

void setMapScrollSpeed(int Speed)
{
	mapScrollSpeed = Speed;
}

int getMapX(void)
{
	if(MyMap)
		return MyMap->Xoffset;
	else
		return 0;
}

int getMapY(void)
{
	if(MyMap)
		return MyMap->Yoffset;
	else
		return 0;
}

void setMapX(int X)
{
	if(MyMap)
		MyMap->Xoffset = X;
}

void setMapY(int Y)
{
	if(MyMap)
		MyMap->Yoffset = Y;
}

int getMapMaxLayer(void)
{
	if(MyMap)
		return MyMap->GetLayers();
	else
		return 0;
}

int getMapLayer(void)
{
	return mapLayer;
}

void setMapLayer(int Layer)
{
	mapLayer = Layer;
}

int MapNextLayer(void)
{
	if(!MyMap) return 0;
	if(mapLayer < MyMap->GetLayers())
		++mapLayer;
	return mapLayer;
}

int MapPrevLayer(void)
{
	if(!MyMap) return 0;
	if(mapLayer > 0)
		--mapLayer;
	return mapLayer;
}

void setMapLeftTile(int Tile)
{
	mapLeftTile = Tile;
}

void setMapRightTile(int Tile)
{
	mapRightTile = Tile;
}

int getMapLeftTile(void)
{
	return mapLeftTile;
}

int getMapRightTile(void)
{
	return mapRightTile;
}

void setMapPath(string path)
{
	MapPath = path;
}

string getMapPath(void)
{
	return MapPath.c_str();
}

int getMapWidth(void)
{
	if(MyMap)
		return MyMap->GetWidth();
	else
		return 0;
}

int getMapHeight(void)
{
	if(MyMap)
		return MyMap->GetHeight();
	else
		return 0;

}

void loadMap(void)
{
	if(MyMap)
		MyMap->Load(MapPath.c_str());
}

void saveMap(void)
{
	if(MyMap)
		MyMap->Save(MapPath.c_str());
}


void resizeMap(int width, int height, int layers)
{
	if(MyMap)
		MyMap->Resize(width, height, layers);
}

void setMapTileset(CRM32Pro_CTile* Tileset)
{
	mapTileset = Tileset;
}

void performMapLogic(void)
{
	// Abort the Logic process if the current Tileset is invalid.
	if(!is_loaded_tileset(mapTileset)) return;
	if(!MyMap) return;

	// Scroll LEFT
	if(getMousex() < 2)
		MyMap->Xoffset -= mapScrollSpeed;

	// Scroll RIGHT
	if(getMousex() > CRM32Pro.screen->w - 2)
		MyMap->Xoffset += mapScrollSpeed;

	// Scroll UP
	if(getMousey() < 2)
		MyMap->Yoffset -= mapScrollSpeed;

	// Scroll DOWN
	if(getMousey() > CRM32Pro.screen->h - 2)
		MyMap->Yoffset += mapScrollSpeed;

	// Scroll speed editting..
	if(isKeypressed(SDLK_F3) && mapScrollSpeed > 0) mapScrollSpeed -= 1;
	if(isKeypressed(SDLK_F2) && mapScrollSpeed < 100) mapScrollSpeed += 1;
	if(isKeypressed(SDLK_F4)) mapScrollSpeed = 10;

	//If mouse is inside the Map GUI Window, do Map Tile Operations.	
	if(isMouseover(256, 0, CRM32Pro.screen->w - 16, CRM32Pro.screen->h - 32))
	{
		// If the left or right mouse button is pressed, draw the tile that was
		// selected by the button that is currently being pressed. If the LEFT SHIFT
		// Key and Left Mouse Button is pressed, we reset that Tile to -1.
		if(isButtonpressed(SDL_BUTTON_LEFT) || isButtonpressed(SDL_BUTTON_RIGHT))
		{															
			int TileWidth;	
			int tileHeight;
			mapTileset->GetOffset(&TileWidth, &tileHeight);
			
			int x = ((int)MyMap->Xoffset + getMousex()) / TileWidth;		// CORRECT!
			int y = ((int)MyMap->Yoffset + getMousey()) / tileHeight;		// !!!
			
			int Tile = 0;
			if(isKeypressed(SDLK_LSHIFT))
				Tile = -1; 
			else
				Tile = (isButtonpressed(SDL_BUTTON_LEFT)) ? 
					mapLeftTile : mapRightTile;
			
			MyMap->SetTile(x, y, mapLayer, Tile);					
		}
				
		// Middle mouse button pressed, scroll map.		
		if(isButtonpressed(SDL_BUTTON_MIDDLE))	
		{
			MyMap->Xoffset += getMousedx();
			MyMap->Yoffset += getMousedy();
		}
		
		// TODO: 
		//
		// If SHIFT & LEFT-BUTTON: Select that tile from the map to left-button paint with. 
		// If SHIFT & RIGHT-BUTTON: Select that tile from the map to right-button paint with. 
		//
	}

}

void performMapRender(void)
{
	// Abort Rendering if the current Tileset is invalid.
	if(!is_loaded_tileset(mapTileset)) return;	
	if(!MyMap) return;
	mapStatus = MyMap->ErrorToStr(MyMap->Render(mapTileset, 0, 0, mapLayer));
}

void initMapWindow(void)
{
	if(!MyMap)
		MyMap = new Map();
	mapTileset = NULL;
}

void closeMapWindow(void)
{
	if(MyMap)
		delete MyMap;
	mapTileset = NULL;
}