/*
	Module:		Map Module
	Programmer:	Michael Scott Stuart
	Version:	1.6.7
	Modified:	10/22/07
	Created:	2/1/07
	Library:	CRM32Pro 4.9.9.0


	10/21/07 - So many code changes!! 
			- Moved map code to a Map Window Module for handling map editting, loading, 
			saving, etc. 
			- Added Tileset Window Module for the Tileset picking menu that is shown. 
			- Added a resource module that contains all of the resources used by the map 
			editor, as well as a resource loader function that initially loads all of the 
			resources used by the map editor.
			- Fixed Prev / Next layer buttons. They now use the IButton Interface.
			- Rearranged the info window so now it is always shown and visually takes up 
			less room.
			- Removed the MapManager namespace. It was pointless. :)
			- Changed the Map::XOffset and YOffset from floats to ints. This makes all 
			previous maps made with the editor non-compatible.
			- The entire map editor now uses one Input Module for keyboard & mouse input
			instead of using all of the various sdl/crm32pro input methods which had thrown
			any kind of input standard out the window.
			- Created a Main Window Module which renders/performs logic for elements of
			the editor that don't fit into any other module. This clears up alot of code.
			- Removed some redundant and performance hitting rectangle drawing that was
			begin done twice behind the Tileset Palette.
	10/03/07 - Revamped Map Manager Module into a Namespace which contains a Map Class. This
			  eases map handling a whole hell of alot!
			- Maps now support multiple Layers.
			TODO: Alot more work is to be done!! Mostly code cleanup and interface fixes.
	03/15/07 - Added advanced map scrolling. Move your mouse to the edges of the screen to scroll the map.
			- Added "F1 FULLSCREEN" text  if windowed, and "F1 WINDOWED" if fullscreen to upper-right menu bar.
			- Removed irrelevant code/modules.
			- Fixed tileset palette window size. It now starts at y-axis 16 instead of 32. (It sounds complicated
			but what it means is now the first row of tiles in the palette are properly selected when clicked on.)
			- Completed the debug information window. This shows the map offsets and the new scroll feature speed.
			You can change the scroll speed using F2 and F3. You can also reset the scroll speed to 10 with F4.
	2/27/07 - Fixed render_map(). (see map_man.h for details). This may be the last version, left at 95% finished. hehe.
	2/06/07 - Changed map rendering in map_man to render an extra row of tiles which weren't being displayed but
			should have been.
			- Added version number display to the bottom menu. (far right in white text)
			- Removed bg image and replaced it with a filled rectangle. (+performance +less resources)
			- Changed output executable to map_editor.exe. (Release & Debug builds)
			- Removed Windows MessageBox displays when a dpf file/tileset or map file fail to load.
	2/03/07 - Extra comments, tileset palette window clean-up. Changed the tileset palette scroll buttons.
	2/02/07 - Cleaned up mouse position and button state code, which helped to remove the fullscreen mouse bug.
			- Finally corrected the tile painting algorithm that calculates what tile the mouse is over. The mouse
			- now accurately paints the tile it is over.
			- Fullscreen mouse works.
	2/01/07 - Fullscreen might have drawing issues with Stardock theme manager. (rob)
	1/31/07 - Lots of GUI changes. 
			- Added Fullscreen/Windowed Hotkey: F1.
			- Fullscreen mouse has a bug. 			
	1/27/07 - Added new feature - RIGHT mouse button tile drawing. (Use right mouse button on tileset palette to
			pick tile for right mouse button tile drawing.) 
			- Really fixed the map loading bug. (YES!)
	1/26/07 - Mmm.. Made bug fixes to map manager module..
	1/25/07 - Made a release build.
			- Tileset picking finished, some bugs - but WONDERFUL nonetheless. :)
	1/24/07 - Maps are now rendered/editable/savable/loadable. :)
			- After conversion from vc express (8.0) to vc7.1, sts editor is now at 50% finished.
			(maps wont edit right now.. but you can load tilesets again) Not only that, but the 
			subsystems got a decent re-working to make up for the lack of design I initially came
			up with. :) This is great because the sts manager now only cares for Tilesets, and it
			automatically tracks a list of all Tilesets in the current DPF. Much nicer. :) On top
			if that, a new map manager now handles all map specific stuff, making it completely
			independant of tilesets - so any map can use any tileset, if you wish for that. 
*/

// 
// MAIN HEADERS/DEFINES/PROTOTYPES IMPLEMENTATION
//
// the include all header
#include "stdafx.h"
#include "Map.h"
#include "ts_win.h"
#include "map_win.h"
#include "main_win.h"

//
// MAIN IMPLEMENTATION DATA
//

//
// MAIN INTERFACE FUNCTIONS
//
 
// Renders the pick tileset window.
void render_picktileset_window(void);
// Performs a Logic step for the pick tileset window.
void logic_picktileset_window(void);
// Render the Menu Window
void render_menu_window(void);
// Logic for the Menu Window
void logic_menu_window(void);
// Renders the Map GUI Window
void render_map_window(void);
// Logic for Map GUI Window
void logic_map_window(void);
// logic for tileset palette window
void logic_tileset_window(void);
// renders the tileset palette window
void render_tileset_window(void);
// editor rendering callback function used by CRM32Pro
void editor_render_callback(int);
// editor logic function. this is the big dog right there, yeeeaaup.
void editor_loop(void);
// The beginning of the program, but not the entry-point.
int editor_main(void);

//
// MAIN ENTRY-POINT IMPLEMENTATION FUNCTIONS
//

// main entry point function for map_editor.
int APIENTRY _tWinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPTSTR    lpCmdLine,
                     int       nCmdShow)
{
	editor_main();
	return 0;
}

//
// IMPLEMENTATION FUNCTIONS
//

// The beginning of the program, but not the entry-point.
int editor_main(void)
{			
	// CRM32Pro

#ifdef _DEBUG
	ILogSystem.Init("map_editor.log", LOG_FILE,LOG_NORMAL,		// Initialize File Logging.
		"map_editor " VERSION);	
#endif
	if(!CRM32Pro.Init(SDL_INIT_VIDEO | SDL_INIT_TIMER))			// Initialize CRM32Pro Core.
	{		
		return 0;
	}	
	ITimeSystem->Init();										// Intialize the Time System. 
	ITimeSystem->SetRate(0, 30);								// (Maintains Render & Logic Rates)	
		
	CRM32Pro.Config.VideoWidth = 800;							// Intialize the video mode/resolution. 
	CRM32Pro.Config.VideoHeight = 600;
	CRM32Pro.Config.VideoRenderer = RENDER_OPENGL;
	CRM32Pro.Config.VideoAccel = ACCEL_HARDSMOOTH;
	CRM32Pro.Config.VideoWindow = 1;							// 1 = windowed, 0 = fullscreen.
	CRM32Pro.SetVideoMode();
	CRM32Pro.SetRenderCallback(editor_render_callback);			// This sends our Render Callback to 
																// CRM32Pro. This function is called every
																// Render frame to render all graphics.

	loadResources();											// Loads and initializes all Editor resources.
																// (GFX/Buttons/Etc..)

	LastButton = -1;											// .. Not used.		


	// Initialize Map Module
	initMapWindow();	

	// Initialize Tileset Window Module
	set_sts_dpf(DEFAULT_DPF);
	setTSWinDPFPath(DEFAULT_DPF);
	setTSWinTSPath(DEFAULT_TILESET);
	loadTSWinTileset();	
	IButton->SetText(DPF_Edit_Box, DEFAULT_DPF);

	// Assign Map Module the Tileset
	setMapTileset(getTSWinTileset());

	// Load the Default Map
	setMapPath(DEFAULT_MAP);
	loadMap();

	string mapxsize, mapysize, maplayers;
	mapxsize += getMapWidth();
	mapysize += getMapHeight();
	maplayers += getMapMaxLayer();
	IButton->SetText(Map_Edit_Box,		(char*)getMapPath().c_str());
	IButton->SetText(Map_XSize,			(char*)mapxsize.c_str());
	IButton->SetText(Map_YSize,			(char*)mapysize.c_str());
	IButton->SetText(Layer_Edit_Box,	(char*)maplayers.c_str());
	
	// Main Editor Loop
	CRM32Pro.Update(&event);									// Call update once for a valid logic state.
	editor_loop();												// Begin the Main Editor Loop.

	// Shutdown the Map Module
	closeMapWindow();

	// Free the Tileset
	freeTSWinTileset();

	delete Font_White_High;
	delete Font_White;	
	CRM32Pro.Quit();											// Shutdown CRM32Pro
	return 0;													// Program Termination
}

// editor loop function. 
void editor_loop(void)
{
	Uint8 done = 0;												// basic loop continue/break flag	
	
	while(!done)
	{      				
		logic_tileset_window();									// Execute logic functions for different sections
		logic_map_window();										// of the editor.
		logic_menu_window();
		logic_picktileset_window();	

		while(CRM32Pro.Update(&event))
		{
			updateInput(&event);

			switch(event.type)
			{
			case SDL_QUIT:
				done = 1;
				break;
			case SDL_KEYDOWN:
				if(event.key.keysym.sym == SDLK_ESCAPE) done = 1;    

				// F1: Fullscreen/Windowed Switch
				if(event.key.keysym.sym == SDLK_F1)
				{								
					(CRM32Pro.Config.VideoWindow) ? 
						CRM32Pro.Config.VideoWindow = 0 : CRM32Pro.Config.VideoWindow = 1;						
					CRM32Pro.SetVideoMode();				
				}
				break;
			default:
				break;
			}
		}
	}	
}

// editor rendering callback function used by CRM32Pro
void editor_render_callback(int bLogicUpdate)
{
	render_map_window();											// Render the Map First.
	render_tileset_window();
	render_menu_window();
	render_picktileset_window();
	LastButton = IButton->Draw();	
}

// Performs a Logic step for the pick tileset window.
void logic_picktileset_window(void)
{
	if(isTSWinActive())
		performTSWinLogic();
}

// Renders the pick tileset window.
void render_picktileset_window(void)
{
	if(isTSWinActive())
		renderTSWin();
}

// Logic for the Menu Window
void logic_menu_window(void)
{	
	// DPF SET BUTTON CLICKED
	if(IButton->GetState(DPF_Set)==B_ACTION)
		if(set_sts_dpf(IButton->GetText(DPF_Edit_Box)))
			setTSWinDPFPath(IButton->GetText(DPF_Edit_Box));

	// TILESET PICK BUTTON CLICKED
	if(IButton->GetState(Tileset_Menu)==B_ACTION)
	{
		setTSWinActive();
	}

	// MAP LOAD BUTTON CLICKED
	if(IButton->GetState(Map_Load)==B_ACTION)
	{
		setMapPath(IButton->GetText(Map_Edit_Box));
		loadMap();
		
		char xstr[10];
		char ystr[10];
		char lstr[10];

		_itoa(getMapWidth(),		xstr, 10);
		_itoa(getMapHeight(),		ystr, 10);
		_itoa(getMapMaxLayer(),		lstr, 10);

		IButton->SetText(Map_XSize,		xstr);
		IButton->SetText(Map_YSize,		ystr);
		IButton->SetText(Layer_Edit_Box,lstr);
	}

	// SAVE MAP BUTTON CLICKED
	if(IButton->GetState(Map_Save)==B_ACTION)
	{
		setMapPath(IButton->GetText(Map_Edit_Box));
		saveMap();
	}

	// RESIZE (SET) MAP BUTTON CLICKED
	if(IButton->GetState(MapSize_Set)==B_ACTION)
	{
		int width = atoi(IButton->GetText(Map_XSize));
		int height = atoi(IButton->GetText(Map_YSize));
		int layers = atoi(IButton->GetText(Layer_Edit_Box));
				
		resizeMap(width, height, layers);
	}

	// PREVIOUS LAYER BUTTON CLICKED
	if(IButton->GetState(Prev_Layer)==B_ACTION)	
	{			
		MapPrevLayer();
	}
 
	// NEXT LAYER BUTTON CLICKED
	if(IButton->GetState(Next_Layer)==B_ACTION)	
	{		
		MapNextLayer();
	}

	// LAYER COUNT EDIT BOX CLICKED
	if(IButton->GetState(Layer_Edit_Box)==B_ACTION)
	{
		int layers = atoi(IButton->GetText(Layer_Edit_Box));	
		resizeMap(getMapWidth(), getMapHeight(), layers);
	}

	/*
		debug information window toggle

		v1.04: with the new GetKeystate and the privatization of keystate I am no \
		longer able to control keypresses. So this will have to do until I think up 
		some way around it.
	*/
	if(isKeypressed(SDLK_F5))
	{
		debug_info = !debug_info;		
	}	
}

// Render the Menu Window
void render_menu_window(void)
{
	renderTopMenu();
	renderBottomMenu();
	renderInfoBar();	
}

// Logic for Map GUI Window
void logic_map_window(void)
{	
	// void map editing if tileset picking is active.
	if(isTSWinActive()) return;

	performMapLogic();
}

// Renders the Map GUI Window
void render_map_window(void)
{	
	setMapTileset(getTSWinTileset());
	performMapRender();
}

// logic for tileset palette window
void logic_tileset_window(void)
{	
	if(!getTSWinTileset()) return;

	if(IButton->GetState(Tileset_Scroll_Up)==B_PRESS)
	{
		if(tileset_scroll_position > 0) 
			--tileset_scroll_position;
	}
	if(IButton->GetState(Tileset_Scroll_Down)==B_PRESS)
	{
		if(tileset_scroll_position < max_scroll_position)
			++tileset_scroll_position;
	}

	//If mouse is inside the Tileset Palette Window, do Tile Selection.
	//if(mx < 256 && (my > 16) && ( my < CRM32Pro.screen->h - 32))
	if(isMouseover(0, 16, TSPALETTE_WIDTH, CRM32Pro.screen->h - 16))	
	{
		if(isButtonpressed(SDL_BUTTON_LEFT) || isButtonpressed(SDL_BUTTON_RIGHT))
		{
			const int scroll_up_h = 
				IButton->GetSurface(Tileset_Scroll_Up, B_NORMAL)->h;
			const int scroll_dn_h = 
				IButton->GetSurface(Tileset_Scroll_Up, B_NORMAL)->h;
			const int scroll_total_h = scroll_up_h + scroll_dn_h;
			int height = CRM32Pro.screen->h;
			int xsize;
			int ysize;
			getTSWinTileset()->GetOffset(&xsize, &ysize);
			int window_x_tiles = TSPALETTE_WIDTH / xsize;
			int window_y_tiles = (height - scroll_total_h) / ysize;
			int tile = ((tileset_scroll_position * window_x_tiles) + 1);
			tile += ( window_x_tiles * 
				((getMousey() - scroll_up_h) / ysize) ) + ( getMousex() / xsize );
			
			(isButtonpressed(SDL_BUTTON_LEFT)) ? 
				setMapLeftTile(tile) : setMapRightTile(tile);
		}
	}	
}

// renders the tileset palette window
void render_tileset_window(void)
{	
	if(!getTSWinTileset()) return;

	// This is potentially dangerous without a NULL check first. :)
	const int scroll_up_h = IButton->GetSurface(Tileset_Scroll_Up, B_NORMAL)->h;
	const int scroll_dn_h = IButton->GetSurface(Tileset_Scroll_Down, B_NORMAL)->h;
	const int scroll_total_h = scroll_up_h + scroll_dn_h;

	// Disable Scroll Buttons if there is no Tileset.
	if(!getTSWinTileset())
	{
		IButton->SetState(Tileset_Scroll_Up, B_OFF);
		IButton->SetState(Tileset_Scroll_Down, B_OFF);
		return;
	}

	int width = CRM32Pro.screen->w;
	int height = CRM32Pro.screen->h;
	int xsize;
	int ysize;
	int surf_xsize;
	int surf_ysize;
	getTSWinTileset()->GetOffset(&xsize, &ysize);
	surf_xsize = getTSWinTileset()->GetSizeX();
	surf_ysize = getTSWinTileset()->GetSizeY();

	SDL_Rect rect;													// We want to clear behind the Tileset Palette
	rect.x = 0;														// so that.. Well, so it looks nice. :)
	rect.y = 0;
	rect.w = TSPALETTE_WIDTH + 2;									// We go two pixels past the width of the 
	rect.h = height;												// palette to give the appearance of a border.
	SDL_FillRect(CRM32Pro.screen, &rect, 0x0A0A0A); 
	
	int tileset_x_tiles = surf_xsize / xsize;						// Total number of tiles on X axis
	int tileset_y_tiles = surf_ysize / ysize;						// Total number of tiles on Y axis
	int window_x_tiles = TSPALETTE_WIDTH / xsize;					// number of tiles to draw on X axis
	int window_y_tiles = (height - scroll_total_h) / ysize;			// number of tiles to draw on Y axis	
	int tile = (tileset_scroll_position * window_x_tiles) + 1;
	max_scroll_position = tileset_y_tiles - window_y_tiles;

	if(IButton->GetState(Tileset_Scroll_Down)==B_OFF)				// Reset the buttons if they're disabled.
		IButton->SetState(Tileset_Scroll_Down, B_NORMAL);
	if(IButton->GetState(Tileset_Scroll_Up)==B_OFF)
		IButton->SetState(Tileset_Scroll_Up, B_NORMAL);

	if(tileset_scroll_position >= max_scroll_position)				// May still scroll down?
	{
		if(IButton->GetState(Tileset_Scroll_Down)!=B_OFF)
			IButton->SetState(Tileset_Scroll_Down, B_OFF);		
	}

	if(tileset_scroll_position < 1)									// May still scroll up?
	{
		if(IButton->GetState(Tileset_Scroll_Up)!=B_OFF)
			IButton->SetState(Tileset_Scroll_Up, B_OFF);		
	}

	for(int y = 0; y < window_y_tiles + 1; ++y)						// Draw the tiles for the Tileset palette.	
		for(int x = 0; x < window_x_tiles; ++x)
		{
			if(tile <= (tileset_x_tiles * tileset_y_tiles))
			{
				SDL_Rect trect;
				trect.x = (x * xsize);
				trect.y = (y * ysize) + scroll_up_h;
				trect.w = (x * xsize) + xsize - 1;
				trect.h = (y * ysize) + ysize + scroll_up_h - 1;

				getTSWinTileset()->SetPosition(trect.x, trect.y);
				getTSWinTileset()->Draw(CRM32Pro.screen, tile);
				
				// These two Rectangles show which tiles are 
				// selected for the mouse buttons.
				if(tile==getMapLeftTile())
					IPrimitives->RectangleColor(CRM32Pro.screen, trect.x,
					trect.y, trect.w, trect.h, 0xFFFFFF);				
				if(tile==getMapRightTile())
					IPrimitives->RectangleColor(CRM32Pro.screen, trect.x,
					trect.y, trect.w, trect.h, 0x00FF00);
			}
			++tile;
		}
}