//
// Module: CRM32Pro SIMPLE TILESET MANAGER
// Author: Michael Scott Stuart
// Notice: Copyright 2007 Michael Scott Stuart
// Status: DEBUG

//
// IMPLEMENTATION
//

//
// HEADERS/DEFINITIONS/LIBS IMPLEMENTATION 
//
#include "sts_man.h"

// <string> class header include
#ifndef _STRING_
#include <string>
#endif // _STRING_
using namespace std;

//
// DEFINES IMPLEMENTATION
//
// Max. number of tilesets to read up to. This number is generous.
#define STS_MAX_TILESETS 128

//
// DATA IMPLEMENTATION
//
// the filename of the dpf to load the tilesets from
char sts_dpf_filename[STS_FILENAME_SIZE];
// names of tilesets found
string sts_tileset_names[STS_MAX_TILESETS];
// total tilesets found in the DPF.
int sts_total_tilesets = 0;
// if properly initialized.
bool sts_dpf_initialized;

//
// FUNCTIONS IMPLEMENTATION
//

// Sets the STS DPF. Builds a Internal Tileset name list to help with accessing Tilesets.
bool set_sts_dpf(char* dpffilename)
{	
	sts_dpf_initialized = false;							// Informs other functions that a DPF is not loaded.
	sprintf(sts_dpf_filename, "%s", dpffilename);			
	int dpfid = IStuffDPF->Open(sts_dpf_filename);			// Apparently Open will return > -1 even if file does
	if(dpfid<0) return false;								// not exist.
	
	sts_total_tilesets = 0;									// This must be reset to 0 every DPF change.
	
	int totalblocks = IStuffDPF->GetNumBlocks(dpfid);		// Get the total number of data blocks in the DPF.
	if(!totalblocks)										// If this is < 1 we obviously opened a bad file and
	{														// Will return false here.
		IStuffDPF->Close(dpfid);
		return false;
	}	
		
	DPF_BlockData dpf_db;							
	if(IStuffDPF->FindFirst(dpfid, &dpf_db))				// Here we intialize the search for Tilesets	
		if(string("TILE2").compare(dpf_db.type)==0)			// If it's a tileset add its name to the array.
		{
			sts_tileset_names[sts_total_tilesets] = dpf_db.name;
			++sts_total_tilesets;
		}	
	
	for(int i = 0; i < totalblocks; ++i)					// Loop through the rest of the data blocks for TILE2 
		if(IStuffDPF->FindNext(dpfid, &dpf_db))				// Tilesets and add what we find to the array.
		{
			if(string("TILE2").compare(dpf_db.type)==0 &&		// Make sure not to add Tilesets if we have met the
				(sts_total_tilesets < STS_MAX_TILESETS))	// limit set by STS_MAX_TILESETS.
			{
				sts_tileset_names[sts_total_tilesets] = dpf_db.name;
				++sts_total_tilesets;
			}
			else
			{												// If we have completely filled the array, clean-up
				// If we have completely filled the array, clean-up 
				if(sts_total_tilesets >= STS_MAX_TILESETS) // and still return true, the operation was successful. 
				{ //->Added by Roberto Prieto 
					// I think you forgot to open and close this condition under the same block, if not, any other 
					// kind of type that will be found on the DPF will break the loop looking for more tileset :) 
					IStuffDPF->Close(dpfid); 
					sts_dpf_initialized = true; 
					return true; 
				} //->Added by Roberto Prieto	
			}
		}	
	
	IStuffDPF->Close(dpfid);								// We're done, close the dpf.
	sts_dpf_initialized = true;								// Notifies other functions that the DPF was loaded.
	return true;
}

// Returns the total number of Tilesets found with set_sts_dpf().
int get_sts_total(void)
{
	return sts_total_tilesets;
}

// Returns the name of a Tileset. Returns NULL if failed.
char* get_sts_name(int sts)
{
	if(sts < 0 || sts >= STS_MAX_TILESETS) return NULL;
	return (char*)sts_tileset_names[sts].c_str();
}

// Returns  true if the tileset is a useable tileset.
bool is_loaded_tileset(CRM32Pro_CTile* ptile)
{
	if(ptile)
	{
		if(ptile->GetSurface()) return true;
	}
	return false;
}


// Loads (and creates) a Tileset and returns it's pointer. Returns NULL if Failed.
// You must delete the Tileset yourself when you are finished with it.
CRM32Pro_CTile* load_tileset(CRM32Pro_CTile* ptile, char* dpfname)
{
	if(!sts_dpf_initialized) return NULL;							// Always make sure a DPF was loaded, if not
																	// there's nothing here to do.	
	for(int i=0; i < sts_total_tilesets; ++i)						// Now find dpfname in the tileset names list.
	{
		if(sts_tileset_names[i].compare(dpfname)==0)				// If the Tileset asked for is in the array,
		{															// we can begin to create that Tileset.
			int dpfid = IStuffDPF->Open(sts_dpf_filename);			// To get the Tileset's data, open the DPF.
			if(dpfid<0) return NULL;	
			DPF_BlockData dpf_db;									// If the data block has a size of 0 or less,
			if(!IStuffDPF->LoadBlock(dpfid,"TILE2",dpfname,&dpf_db))// something went wrong and we must NULL out.
			{
				IStuffDPF->Close(dpfid);
				return NULL;
			}

			SDL_Surface* tsurf = NULL;								// Create some vars that will hold the data
			long surf_x_size = 0;									// used to create the Tileset's Surface (image),
			long surf_y_size = 0;									// and then to create the Tileset itself.
			long pitch = 0;											// sdl note: pitch is width of image in bytes
			long x_off = 0;
			long y_off = 0;
			SDL_PixelFormat surf_pf;			
			int surf_pixels_size = dpf_db.size - 128;							
			char* surf_pixels = new char[ (surf_pixels_size) ];	

			// The following is a _guess_ on the methods of: how to get the data required to
			// create a Tileset, and: How to, in return, create that Tileset.

			/*CRM32Pro_*/	memcpy(&surf_x_size, &dpf_db.data[0], 4);
			/*CRM32Pro_*/	memcpy(&surf_y_size, &dpf_db.data[4], 4);
			/*CRM32Pro_*/	memcpy(&x_off, &dpf_db.data[20], 4);
			/*CRM32Pro_*/	memcpy(&surf_pf, &dpf_db.data[24], 40);

							pitch = surf_pf.BytesPerPixel * surf_x_size;
							//pitch = surf_pixels_size / surf_y_size;

			/*CRM32Pro_*/	memcpy(surf_pixels, &dpf_db.data[64], surf_pixels_size );
			/*CRM32Pro_*/	memcpy(&y_off, &dpf_db.data[64 + surf_pixels_size], 4);
			
			// Create SDL_Surface for Tileset. 
			tsurf = SDL_CreateRGBSurfaceFrom(surf_pixels, surf_x_size, surf_y_size, surf_pf.BitsPerPixel, 
				pitch, surf_pf.Rmask, surf_pf.Gmask, surf_pf.Bmask, surf_pf.Amask);
			if(!tsurf)												// If we failed to create the SDL_Surface for
			{														// the Tileset we have to return NULL, there's
				delete [] surf_pixels;								// nothing else we can do to be successful.
				IStuffDPF->FreeBlock(&dpf_db);						// Frees the loaded block from memory.
				IStuffDPF->Close(dpfid);
				return NULL;
			}

			if(!ptile)
				CRM32Pro_CTile* ptile = new CRM32Pro_CTile();		// Now we can create the Tileset, and set 
			ptile->Create(tsurf, dpfname);							// it's data we just obtained.
			ptile->SetTileSet(1, x_off, y_off);						// Set Tileset flag and size per tile.
			IStuffDPF->FreeBlock(&dpf_db);							// Frees the loaded block from memory.
			IStuffDPF->Close(dpfid);
			delete [] surf_pixels;									// We can free the pixels and the surface 
			CRM32Pro.FreeSurface(tsurf);							// Because it seems CRM32Pro_CTile copies
																	// that data instead of using it directly.
			return ptile;
		}
	}		
	return NULL;													// We've gone this far only if the Tileset
																	// dpfname we were searching for was not found.
}

// Module: CRM32Pro SIMPLE TILESET MANAGER